@@ -32,6 +32,15 @@
 #include "midgard_view.h"
 #include "midgard_timestamp.h"
 #include "midgard_metadata.h"
+#include "midgard_workspace.h"
+#include "midgard_repligard.h"
+#include "midgard_base_abstract.h"
+#include "midgard_base_interface.h"
+#include "midgard_base_mixin.h"
+#include "midgard_query_executor.h"
+#include "midgard_query_select.h"
+
+#include <locale.h>
 
 #include <libgda/libgda.h>
 
@@ -174,6 +183,35 @@ midgard_init()
 	MidgardMetadata *m = g_object_new (MIDGARD_TYPE_METADATA, NULL);
 	g_object_unref (m);
 
+	type = MIDGARD_TYPE_WORKSPACE;
+	g_assert (type != 0);
+	g_type_class_ref (type);
+
+	type = MIDGARD_TYPE_WORKSPACE_STORAGE;
+	g_assert (type != 0);
+
+	type = MIDGARD_TYPE_REPLIGARD;
+	g_assert (type != 0);
+	g_type_class_ref (type);
+
+	type = MIDGARD_TYPE_BASE_ABSTRACT;
+	g_assert (type != 0);
+	g_type_class_ref (type);
+
+	type = MIDGARD_TYPE_QUERY_EXECUTOR;
+	g_assert (type != 0);
+	g_type_class_ref (type);
+
+	type = MIDGARD_TYPE_QUERY_SELECT;
+	g_assert (type != 0);
+	g_type_class_ref (type);
+
+	type = MIDGARD_TYPE_BASE_INTERFACE;
+	g_assert (type != 0);
+
+	type = MIDGARD_TYPE_BASE_MIXIN;
+	g_assert (type != 0);
+
 	/* Register transform function explicitly, we need own routine */
 	g_value_register_transform_func (G_TYPE_STRING, G_TYPE_FLOAT, __transform_string_to_float);
   	g_value_register_transform_func (G_TYPE_STRING, G_TYPE_BOOLEAN, __transform_string_to_boolean);
@@ -223,4 +261,24 @@ midgard_close(void)
 	klass = g_type_class_peek (MIDGARD_TYPE_METADATA);
 	if (klass)
 		g_type_class_unref (klass);
+
+	klass = g_type_class_peek (MIDGARD_TYPE_WORKSPACE);
+	if (klass)
+		g_type_class_unref (klass);
+
+	klass = g_type_class_peek (MIDGARD_TYPE_REPLIGARD);
+	if (klass)
+		g_type_class_unref (klass);
+
+	klass = g_type_class_peek (MIDGARD_TYPE_BASE_ABSTRACT);
+	if (klass)
+		g_type_class_unref (klass);
+
+	klass = g_type_class_peek (MIDGARD_TYPE_QUERY_EXECUTOR);
+	if (klass)
+		g_type_class_unref (klass);
+
+	klass = g_type_class_peek (MIDGARD_TYPE_QUERY_SELECT);
+	if (klass)
+		g_type_class_unref (klass);
 }
