@@ -110,6 +110,79 @@ gchar *midgard_core_query_where_guid(
 	return g_string_free(where, FALSE);
 }
 
+gint 
+midgard_core_query_execute_statement (MidgardConnection *mgd, const gchar *query, gboolean ignore_error, gchar *holder_id, ...)
+{
+	g_return_val_if_fail (mgd != NULL, -1);
+	g_return_val_if_fail (query != NULL, -1);
+	g_return_val_if_fail (holder_id != NULL, -1);
+
+	GdaStatement *stmt;
+	GError *lerror = NULL;
+	gint number = 0;
+
+	/* create a parser if necessary */
+	if (!mgd->priv->parser) {
+		mgd->priv->parser = gda_connection_create_parser (mgd->priv->connection);
+		if (!mgd->priv->parser)
+			mgd->priv->parser = gda_sql_parser_new ();
+	}
+
+	/* create a new statement */
+	stmt = gda_sql_parser_parse_string (mgd->priv->parser, query, NULL, &lerror);
+	if (stmt) {
+		gchar *tmp_holder = holder_id;
+		va_list args;
+		const gchar *value_arg = NULL;
+		GdaHolder *gda_holder = NULL;
+
+		GdaSet *params;
+	       	if (!gda_statement_get_parameters (stmt, &params, &lerror)) {
+			g_object_unref (stmt);
+			g_warning ("Error while preparing statement '%s': %s\n", query, lerror && lerror->message ? lerror->message : "No detail");
+			if (lerror)
+				g_error_free (lerror);
+			return -1;
+		}
+
+		va_start (args, holder_id);
+
+		while (tmp_holder != NULL) {
+       			value_arg = va_arg (args, const gchar*);
+			g_return_val_if_fail (value_arg != NULL, -1);
+			gda_holder = gda_set_get_holder (params, (const gchar *)tmp_holder);
+			g_return_val_if_fail (gda_holder != NULL, -1);
+			gda_holder_set_value_str (gda_holder, NULL, value_arg, &lerror);
+			if (lerror) {
+				g_warning ("Failed to set '%s' holder value (%s) ", tmp_holder, value_arg);
+				g_error_free (lerror);
+			}
+			tmp_holder = va_arg (args, gchar*);
+		}
+		va_end (args);
+
+		if (MGD_CNC_DEBUG (mgd)) {
+			gchar *debug_query = gda_connection_statement_to_sql (mgd->priv->connection, stmt, params, GDA_STATEMENT_SQL_PRETTY, NULL, NULL);
+			g_debug ("%s", debug_query);
+			g_free (debug_query);
+		}
+			
+		number = gda_connection_statement_execute_non_select (mgd->priv->connection, stmt, params, NULL, &lerror);
+		g_object_unref (stmt);
+	}
+
+	if(!ignore_error) {
+		if (!stmt || (number == -1)) {
+			g_warning ("Error executing '%s': %s\n", query, lerror && lerror->message ? lerror->message : "No detail");
+		}
+	}
+
+	if (lerror)
+		g_error_free (lerror);
+
+	return (number);
+}
+
 gint 
 midgard_core_query_execute (MidgardConnection *mgd, const gchar *query, gboolean ignore_error)
 {
@@ -563,7 +636,7 @@ midgard_core_query_get_dbobject_model (MidgardConnection *mgd, MidgardDBObjectCl
 	gfloat pfloat; 									\
 	MidgardTimestamp *mt;								\
 	gboolean pbool;									\
-	gchar *pname = pspecs[i]->name; 						\
+	const gchar *pname = pspecs[i]->name; 						\
 	switch (G_TYPE_FUNDAMENTAL (pspecs[i]->value_type)) { 				\
 		case G_TYPE_STRING:							\
 			g_object_get (object, pname, &pstr, NULL);			\
@@ -1610,8 +1683,8 @@ gboolean midgard_core_query_create_metadata_columns(
 		mdc = midgard_core_dbcolumn_new();
 		mdc->table_name = tablename;
 		mdc->column_name = bool_columns[i];
-		mdc->gtype = MGD_TYPE_INT;
-		mdc->dbtype = "int";
+		mdc->gtype = G_TYPE_BOOLEAN;
+		mdc->dbtype = "bool";
 		mdc->dvalue = "0";
 
 		rv = midgard_core_query_add_column(mgd, mdc);
@@ -2166,7 +2239,7 @@ __exclude_deleted_constraint (MidgardQueryExecutor *executor, const gchar *table
 
 	GdaSqlExpr *expr = gda_sql_expr_new (GDA_SQL_ANY_PART (operation));
 	expr->value = gda_value_new (G_TYPE_STRING);
-	g_value_set_string (expr->value, "t1.metadata_deleted = 0");
+	g_value_set_string (expr->value, "t1.metadata_deleted = FALSE");
 	operation->operands = g_slist_append (operation->operands, expr);
 }
 
@@ -2479,12 +2552,12 @@ midgard_core_query_get_objects (MidgardConnection *mgd, const gchar *classname,
 {
 	if (mgd == NULL) {
 		g_set_error (error, MIDGARD_GENERIC_ERROR, MGD_ERR_INTERNAL, "Expected MidgardConnection. Can not get object.");
-		return;
+		return NULL;
 	}
 
 	if (classname == NULL) {
 		g_set_error (error, MIDGARD_GENERIC_ERROR, MGD_ERR_INTERNAL, "Expected class name. Can not get object.");
-		return;
+		return NULL;
 	}
 
 	GSList *arg_list = NULL;
@@ -2511,7 +2584,7 @@ midgard_core_query_get_objects (MidgardConnection *mgd, const gchar *classname,
 
 	if (arg_list == NULL) {
 		g_set_error (error, MIDGARD_GENERIC_ERROR, MGD_ERR_INTERNAL, "Expected at last one constraint to get object");
-		return;
+		return NULL;
 	}
 
 	/* We need either group or one constraint */
