@@ -133,13 +133,14 @@ midgard_reflector_object_get_property_parent (const gchar *classname)
  * Returns newly allocated, children ( in midgard tree ) classes' names. 
  * Returned array should be freed if no longer needed without freeing array's elements. 
  *
- * Returns: (transfer container): array of strings or %NULL.
+ * Returns: (array length=n_children) (transfer container): array of strings or %NULL.
  * Since: 10.05
  */  
 gchar**
 midgard_reflector_object_list_children (const gchar *classname, guint *n_children)
 {
 	g_return_val_if_fail (classname != NULL, NULL);
+	g_return_val_if_fail (n_children != NULL, NULL);
 
 	_GET_CLASS_BY_NAME (classname, NULL);
 	_GET_TYPE_ATTR (klass);
@@ -147,16 +148,22 @@ midgard_reflector_object_list_children (const gchar *classname, guint *n_childre
 	if (!type_attr->children)
 		return NULL;
 
-	 GSList *list;
-	 GSList *children = type_attr->children;
+	 GSList *list; 
 	 guint i = 0;
+	 guint length = g_slist_length (type_attr->children);
+	 *n_children = length;
 
-	 gchar **children_class = g_new (gchar*, g_slist_length (children));
+	 /* possible? */
+	 if (length == 0)
+		 return NULL;
 
-	 for (list = children; list != NULL; list = list->next, i++) 
+	 gchar **children_class = g_new (gchar*, length + 1);
+
+	 for (list = type_attr->children; list != NULL; list = list->next, i++) 
 		 children_class[i] = (gchar *) list->data;
 
-	 *n_children = i;
+	 *n_children = length;
+	 children_class[length] = NULL;
 	 
 	 return children_class;
 }
@@ -315,7 +322,7 @@ midgard_reflector_object_list_defined_properties (const gchar *classname, guint
 	guint i;
 	for (i = 0; i < *n_prop; i++)
 	{
-		names[i] = pspecs[i]->name;
+		names[i] = (gchar*) pspecs[i]->name;
 	}
 
 	g_free(pspecs);
