@@ -33,6 +33,11 @@
 #include "guid.h"
 #include "midgard_core_object_class.h"
 #include "midgard_core_connection.h"
+#include "midgard_query_value.h"
+#include "midgard_query_constraint.h"
+#include "midgard_query_constraint_group.h"
+#include "midgard_query_select.h"
+#include "midgard_executable.h"
 
 #if HAVE_CRYPT_H
 # include <crypt.h>
@@ -203,7 +208,7 @@ __convert_to_storage_parameters (MidgardConnection *mgd, guint n_params, const G
  * </para></listitem>
  * </itemizedlist>
  *
- * Returns: new #MidgardUser instance or %NULL in case of failure
+ * Returns: (transfer full): new #MidgardUser instance or %NULL in case of failure
  * 
  * Since 9.09
  */
@@ -282,19 +287,13 @@ __midgard_user_get (MidgardConnection *mgd, guint n_params, const GParameter *pa
  * Cases to return %NULL:
  * <itemizedlist> 
  * <listitem><para>
- * There's attempt to query users in anonymous mode or logged in user is not admin (MGD_ERR_ACCESS_DENIED)
- * </para></listitem>
- * <listitem><para>
- * 'login' or 'authtype' properties do not exist in given parameters (MGD_ERR_INVALID_PROPERTY_VALUE)
- * </para></listitem>
- * <listitem><para>
  * There are no user objects which match given parameters (MGD_ERR_NOT_EXISTS)
  * </para></listitem>
  * </itemizedlist>
  *
  * Returned array should be freed (g_free()) when no longer needed.
  * 
- * Returns: newly allocated and NULL terminated array of #MidgardUser objects or %NULL
+ * Returns: (transfer full): newly allocated and NULL terminated array of #MidgardUser objects or %NULL
  * 
  * Since 9.09
  */
@@ -310,67 +309,70 @@ __midgard_user_query (MidgardConnection *mgd, guint n_params, const GParameter *
 {
 	g_return_val_if_fail (mgd != NULL, NULL);
 	g_return_val_if_fail (parameters != NULL, NULL);
+	MidgardUser **result = NULL;
+
+	if (n_params == 0) {
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_INTERNAL, "Expected one parameter (at least) to get user object");
+		return result;
+	}
 
 	MIDGARD_ERRNO_SET (mgd, MGD_ERR_OK);
 
-	/* Deny queries for non admin user*/
-	MidgardUser *cuser = midgard_connection_get_user (mgd);
-	if (!cuser || (cuser && !midgard_user_is_admin (cuser))) {
-
-		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_ACCESS_DENIED, "Not allowed to query users for non admin users.");
-		return NULL;
-	}
-
 	guint i;
+	GSList *constraints = NULL;
+	GSList *objects_list = NULL;
 	MidgardUserClass *klass = g_type_class_peek (MIDGARD_TYPE_USER);
 
-	/* Validate properties and find required ones */
-	if (!__validate_parameters(mgd, n_params, parameters, 1)) 
-		return NULL;
-
-	/* Create new parameters. We need to pass column names instead of properties */
-	GValue idval = {0, };
-	g_value_init (&idval, G_TYPE_UINT);
-	GParameter *tbl_params = __convert_to_storage_parameters (mgd, n_params, parameters, klass, idval);
-	if (!tbl_params) {
-		
-		g_value_unset (&idval);
-		return NULL;
-	}
-	GdaDataModel *model = midgard_core_query_get_dbobject_model (mgd, MIDGARD_DBOBJECT_CLASS (klass), n_params, tbl_params);
-
-	g_free (tbl_params);
-	g_value_unset (&idval);
-
-	if (!model) {
-
-		MIDGARD_ERRNO_SET (mgd, MGD_ERR_NOT_EXISTS);
-		return NULL;
+	for (i = 0; i < n_params; i++) {
+		MidgardQueryProperty *mqp = midgard_query_property_new (parameters[i].name, NULL);
+		objects_list = g_slist_prepend (objects_list, (gpointer) mqp);
+		MidgardQueryValue *mqv = midgard_query_value_create_with_value (((const GValue *) &parameters[i].value));
+		objects_list = g_slist_prepend (objects_list, (gpointer) mqv);
+		MidgardQueryConstraint *mqc = midgard_query_constraint_new (mqp, "=", MIDGARD_QUERY_HOLDER (mqv), NULL);
+		objects_list = g_slist_prepend (objects_list, (gpointer) mqc);
+		constraints = g_slist_prepend (constraints, mqc);
 	}
 
-	guint rows = gda_data_model_get_n_rows(model);
-	if(rows == 0) {
-
-		if (model) {
-			g_object_unref(model);
-			model = NULL;
+	/* We need either group or one constraint */
+	MidgardQueryConstraintSimple *constraint = NULL;
+	GSList *l = NULL;
+	if (g_slist_length (constraints) > 1) {
+		MidgardQueryConstraintGroup *cgroup = midgard_query_constraint_group_new ();
+		for (l = constraints; l != NULL; l = l->next) {
+			midgard_query_constraint_group_add_constraint (cgroup, (MidgardQueryConstraintSimple *) l->data);
 		}
-
-		MIDGARD_ERRNO_SET (mgd, MGD_ERR_NOT_EXISTS);
-		return NULL;
+		constraint = (MidgardQueryConstraintSimple*) cgroup;
+	} else {
+		constraint = (MidgardQueryConstraintSimple *) constraints->data;
 	}
+	g_slist_free (constraints);
 
-	MidgardUser **users = g_new (MidgardUser*, rows+1);
-
-	for (i = 0; i < rows; i++) {
+	MidgardQueryStorage *storage = midgard_query_storage_new (g_type_name (MIDGARD_TYPE_USER));
+	MidgardQuerySelect *select = midgard_query_select_new (mgd, storage);
+	midgard_query_executor_set_constraint (MIDGARD_QUERY_EXECUTOR (select), constraint);
+	midgard_query_select_toggle_read_only (select, FALSE);
 	
-		users[i] = g_object_new (MIDGARD_TYPE_USER, "connection", mgd, NULL);
-		MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->__set_from_sql (MIDGARD_DBOBJECT (users[i]), model, i);
+	GError *err = NULL;
+	midgard_executable_execute (MIDGARD_EXECUTABLE (select), &err);
+	if (err) {
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_INTERNAL, 
+				"Failed to query user. %s", err && err->message ? err->message : "Unknown reason");
+		g_clear_error (&err);
+		goto free_objects_and_return;
 	}
 
-	users[i] = NULL;
+	guint n_objects;
+	result = (MidgardUser **) midgard_query_select_list_objects (select, &n_objects);
 
-	return users;
+free_objects_and_return:
+	for (l = objects_list; l != NULL; l = l->next) {
+		g_object_unref ((GObject*) l->data);
+	}
+        g_slist_free (objects_list);
+        g_object_unref (storage);
+        g_object_unref (select);
+
+        return result;
 }	
 
 /** 
@@ -619,7 +621,7 @@ __midgard_user_update (MidgardUser *self)
 		}
 	}
 
-	if (midgard_core_query_update_dbobject_record (MIDGARD_DBOBJECT (self)))
+	if (midgard_core_query_update_dbobject_record (MIDGARD_DBOBJECT (self), NULL) )
 		return TRUE;
 
 	MIDGARD_ERRNO_SET (mgd, MGD_ERR_INTERNAL);
@@ -999,7 +1001,7 @@ static void __set_from_sql(MidgardDBObject *object,
 	/* ACTIVE */
 	value = midgard_data_model_get_value_at_col_name (model, "active", row);
 	if (value) {
-		self->priv->active = g_value_get_boolean (value);
+		MIDGARD_GET_BOOLEAN_FROM_VALUE (self->priv->active, value);
 	}
 
 	/* AUTH TYPE */
@@ -1023,6 +1025,95 @@ static void __set_from_sql(MidgardDBObject *object,
 	}
 }
 
+xmlNode *__lookup_node(xmlNode *node, const gchar *name)
+{
+	xmlNode *cur = NULL;
+	
+	for (cur = node; cur; cur = cur->next) {
+		if (cur->type == XML_ELEMENT_NODE) {
+			if (g_str_equal (cur->name, name))
+				return cur;
+    		}
+	}
+	return NULL;
+}
+
+static void 
+__set_from_xml_node (MidgardDBObject *object, xmlNode *node)
+{
+	g_assert (object != NULL);
+	g_assert (node != NULL);
+
+	MidgardUser *self = MIDGARD_USER (object);
+	xmlNode *lnode = NULL;
+	gchar *content;
+
+	/* guid */
+	lnode = __lookup_node (node, "guid");
+	g_free ((gchar*) MIDGARD_DBOBJECT (self)->dbpriv->guid);
+	//self->dbpriv->guid = ;
+
+	/* login */
+	lnode = __lookup_node (node, "login");
+	content = (gchar *)xmlNodeGetContent (lnode);
+	if (content != NULL) {
+		g_free (self->priv->login);
+		self->priv->login = g_strdup (content);
+		g_free (content);
+	}
+
+	/* password */
+	lnode = __lookup_node (node, "password");
+	content = (gchar *)xmlNodeGetContent (lnode);
+	if (content != NULL) {
+		g_free (self->priv->password);
+		self->priv->password = g_strdup (content);
+		g_free (content);
+	}
+
+	/* active */
+	lnode = __lookup_node (node, "active");
+	content = (gchar *)xmlNodeGetContent (lnode);
+	if (content != NULL) {
+		self->priv->active = (gboolean)atoi (content);
+		g_free (content);
+	}
+
+	/* authtype */
+	lnode = __lookup_node (node, "authtype");
+	content = (gchar *)xmlNodeGetContent (lnode);
+	if (content != NULL) {
+		g_free (self->priv->auth_type);
+		self->priv->auth_type = g_strdup (content);
+		g_free (content);
+	}
+
+	/* authtypeid */
+	lnode = __lookup_node (node, "authtypeid");
+	content = (gchar *)xmlNodeGetContent (lnode);
+	if (content != NULL) {
+		self->priv->auth_type_id = atoi (content);
+		g_free (content);
+	}
+
+	/* usertype */
+	lnode = __lookup_node (node, "usertype");
+	content = (gchar *)xmlNodeGetContent (lnode);
+	if (content != NULL) {
+		self->priv->user_type = atoi (content);
+		g_free (content);
+	}
+
+	/* person */
+	lnode = __lookup_node (node, "person");
+	content = (gchar *)xmlNodeGetContent (lnode);
+	if (content != NULL) {
+		g_free (self->priv->person_guid);
+		self->priv->person_guid = g_strdup (content);
+		g_free (content);
+	}
+}
+
 /**
  * midgard_user_set_person:
  * @self: #MidgardUser instance
@@ -1031,7 +1122,7 @@ static void __set_from_sql(MidgardDBObject *object,
  * Associates given #MidgardObject person with @self #MidgardUser.
  * Sets person property and updates user storage record.
  *
- * #MidgardUser @self takes ownership of the given #MidgardPerson reference, 
+ * #MidgardUser @self takes ownership of the given #MidgardObject ('midgard_person' type) reference, 
  * and increases person's object reference count. 
  * 
  * See midgard_user_update() for returned error details.
@@ -1193,7 +1284,7 @@ _midgard_user_set_property (GObject *object, guint property_id,
 			break;
 
 		case MIDGARD_USER_ACTIVE:
-			self->priv->active = g_value_get_boolean (value);
+			MIDGARD_GET_BOOLEAN_FROM_VALUE (self->priv->active, value);
 			break;
 
 		case MIDGARD_USER_AUTH_TYPE:
@@ -1321,6 +1412,57 @@ _user_storage_delete (MidgardConnection *mgd, MidgardDBObjectClass *klass)
 	return FALSE;
 }
 
+static void
+__set_from_data_model (MidgardDBObject *self, GdaDataModel *model, gint row, guint column_id)
+{
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (model != NULL);
+	g_return_if_fail (row > -1);
+	
+	GError *error = NULL;
+	const GValue *value;
+
+	/* guid */
+	value = gda_data_model_get_value_at (model, column_id, row, &error);
+	if (!value) {
+		g_warning ("Failed to get guid value: %s", error && error->message ? error->message : "Unknown reason");
+		if (error)
+			g_clear_error (&error);
+	} else {
+		g_free ((gchar *)MGD_OBJECT_GUID (self));
+		MGD_OBJECT_GUID (self) = g_value_dup_string (value);
+	}
+	if (error) g_clear_error (&error);
+	
+	/* FIXME, add column indexes (or constants) for easier data fetching, COL_ID_PERSON instead of 7, etc */ 
+	/* person */
+	value = gda_data_model_get_value_at (model, 7, row, &error);
+	if (!value) {
+		g_warning ("Failed to get person value: %s", error && error->message ? error->message : "Unknown reason");
+		if (error)
+			g_clear_error (&error);
+	} else {
+		g_free(MIDGARD_USER(self)->priv->person_guid);
+		MIDGARD_USER(self)->priv->person_guid = g_value_dup_string (value);
+	}
+	if (error) g_clear_error (&error);
+
+	/* authtypeid */
+	value = gda_data_model_get_value_at (model, 5, row, &error);
+	if (!value) {
+		g_warning ("Failed to get auth type id  value: %s", error && error->message ? error->message : "Unknown reason");
+		if (error)
+			g_clear_error (&error);
+	} else {
+		MIDGARD_USER(self)->priv->auth_type_id = g_value_get_int (value);
+	}
+	if (error) g_clear_error (&error);
+
+	column_id++;
+	
+	MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->set_from_data_model (self, model, row, column_id);
+}
+
 static void _midgard_user_class_init(
 		gpointer g_class, gpointer g_class_data)
 {
@@ -1363,6 +1505,7 @@ static void _midgard_user_class_init(
 			MIDGARD_USER_GUID,
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
+	prop_attr->name = g_strdup (property_name);
 	prop_attr->gtype = MGD_TYPE_GUID;
 	prop_attr->field = g_strdup(property_name);
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
@@ -1382,6 +1525,7 @@ static void _midgard_user_class_init(
 			MIDGARD_USER_LOGIN,
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
+	prop_attr->name = g_strdup (property_name);
 	prop_attr->gtype = MGD_TYPE_STRING;
 	prop_attr->field = g_strdup(property_name);
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
@@ -1401,6 +1545,7 @@ static void _midgard_user_class_init(
 			MIDGARD_USER_PASS,
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
+	prop_attr->name = g_strdup (property_name);
 	prop_attr->gtype = MGD_TYPE_STRING;
 	prop_attr->field = g_strdup(property_name);
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
@@ -1419,6 +1564,7 @@ static void _midgard_user_class_init(
 			MIDGARD_USER_PERSON_GUID,
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
+	prop_attr->name = g_strdup (property_name);
 	prop_attr->gtype = MGD_TYPE_GUID;
 	prop_attr->field = g_strdup("person_guid");
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
@@ -1439,6 +1585,7 @@ static void _midgard_user_class_init(
 			MIDGARD_USER_ACTIVE,
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
+	prop_attr->name = g_strdup (property_name);
 	prop_attr->gtype = MGD_TYPE_BOOLEAN;
 	prop_attr->field = g_strdup(property_name);
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
@@ -1457,6 +1604,7 @@ static void _midgard_user_class_init(
 			MIDGARD_USER_AUTH_TYPE,
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
+	prop_attr->name = g_strdup (property_name);
 	prop_attr->gtype = MGD_TYPE_STRING;
 	prop_attr->field = g_strdup("auth_type");
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
@@ -1474,6 +1622,7 @@ static void _midgard_user_class_init(
 			MIDGARD_USER_AUTH_TYPE_ID,
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
+	prop_attr->name = g_strdup (property_name);
 	prop_attr->gtype = MGD_TYPE_UINT;
 	prop_attr->field = g_strdup("auth_type_id");
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
@@ -1492,6 +1641,7 @@ static void _midgard_user_class_init(
 			MIDGARD_USER_TYPE,
 			pspec);
 	prop_attr = midgard_core_schema_type_property_attr_new();
+	prop_attr->name = g_strdup (property_name);
 	prop_attr->gtype = MGD_TYPE_UINT;
 	prop_attr->field = g_strdup("user_type");
 	prop_attr->table = g_strdup(MIDGARD_USER_TABLE);
@@ -1500,20 +1650,27 @@ static void _midgard_user_class_init(
 	g_hash_table_insert(type_attr->prophash, g_strdup((gchar *)property_name), prop_attr);
 	type_attr->_properties_list = g_slist_append (type_attr->_properties_list, property_name);
 
-	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv = g_new(MidgardDBObjectPrivate, 1);
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv = midgard_core_dbobject_private_new ();
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data = type_attr;
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data->table = g_strdup(MIDGARD_USER_TABLE);
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data->tables = g_strdup(MIDGARD_USER_TABLE);
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->has_metadata = FALSE;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->uses_workspace = FALSE;
 
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->add_fields_to_select_statement = MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->add_fields_to_select_statement;
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->__set_from_sql = __set_from_sql;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_from_xml_node = __set_from_xml_node;
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_from_sql = NULL;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_from_data_model = __set_from_data_model;
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->create_storage = midgard_core_query_create_class_storage;	
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->update_storage = midgard_core_query_update_class_storage;
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_exists = _user_storage_exists; 
 	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->delete_storage = _user_storage_delete;
-	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_statement_insert = MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->set_statement_insert;
-	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_statement_update = MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->set_statement_update;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_static_sql_select = NULL;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->get_statement_insert = MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->get_statement_insert;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->get_statement_insert_params = MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->get_statement_insert_params;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->get_statement_update = MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->get_statement_update;
+	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->get_statement_update_params = MIDGARD_DBOBJECT_CLASS (__parent_class)->dbpriv->get_statement_update_params;
 
 	type_attr->params = midgard_core_dbobject_class_list_properties (MIDGARD_DBOBJECT_CLASS (klass), &type_attr->num_properties);	
 	guint i;
@@ -1528,10 +1685,6 @@ static void _midgard_user_class_init(
 	}
 	type_attr->sql_select_full = g_strdup (sql_full->str);
 	g_string_free (sql_full, TRUE);
-
-	 /* Initialize persistent statements */
-	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_statement_insert (MIDGARD_DBOBJECT_CLASS (klass));
-	MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->set_statement_update (MIDGARD_DBOBJECT_CLASS (klass));
 }
 
 static void _midgard_user_instance_init(
