@@ -17,7 +17,6 @@
  */
 
 #include "src/midgard_error.h"
-#include "src/midgard_object_class.h"
 #include "src/midgard_user.h"
 #include "src/midgard_object_parameter.h"
 #include "src/midgard_utils.h"
@@ -28,7 +27,7 @@
 #define G_LOG_DOMAIN "Schema"
 #endif
 
-static void _test_me(MidgardConnection *mgd, MgdObject *object)
+static void _test_me(MidgardConnection *mgd, MidgardObject *object)
 {
 	/* This is anonymous mode now. 
 	 * It should be supported by SG account in config file later.
@@ -101,7 +100,7 @@ static void _test_me(MidgardConnection *mgd, MgdObject *object)
 		gchar *guid;
 		g_object_get(G_OBJECT(object), "guid", &guid, NULL);
 		g_value_set_string(&gval, guid);
-		MgdObject *new = midgard_object_new(mgd, typename, &gval);
+		MidgardObject *new = midgard_object_new(mgd, typename, &gval);
 		g_value_unset(&gval);
 		
 		if (new == NULL) {
@@ -116,7 +115,7 @@ static void _test_me(MidgardConnection *mgd, MgdObject *object)
 		
 		g_free(guid);
 
-		if (midgard_object_delete(object)) {
+		if (midgard_object_delete(object, TRUE)) {
 
 			g_message("Object '%s' delete : OK", typename);
 
@@ -197,9 +196,6 @@ main (int argc, char **argv)
 			"midgardusername", &mgdusername, 
 			"midgardpassword", &mgdpassword,
 			NULL);
-
-	if ((mgdusername != NULL) && (mgdpassword != NULL))
-		midgard_user_auth(mgd, mgdusername, mgdpassword, NULL, FALSE);
 		
 	/* Create basic tables first */
 	if (tablecreate)
@@ -210,24 +206,22 @@ main (int argc, char **argv)
 		typename = g_type_name(all_types[i]);
 
 		if (g_str_equal(typename, "midgard_parameter")) {
-			MgdObject *param = midgard_object_new(mgd, typename, NULL);
+			MidgardObject *param = midgard_object_new(mgd, typename, NULL);
 			g_object_unref(param);
 			g_message("midgard_parameter ignored");
 			continue;
 		}
 
 		g_message("Type %s", typename);
-		MgdObject *object = midgard_object_new(mgd, typename, NULL);
-
-		MidgardObjectClass *klass = MIDGARD_OBJECT_GET_CLASS_BY_NAME(typename);
+		MidgardObject *object = midgard_object_new(mgd, typename, NULL);
 
 		/* Create tables */
 		if (tablecreate) 
-			midgard_storage_create_class_storage (mgd, MIDGARD_DBOBJECT_CLASS (klass));
+			midgard_storage_create (mgd, typename);
 
 		/* Update tables */
 		if (tableupdate) 
-			midgard_storage_update_class_storage (mgd, MIDGARD_DBOBJECT_CLASS (klass));
+			midgard_storage_update (mgd, typename);
 			
 		/* Test objects */
 		if (testunit)
